set linesize 80
set pagesize 1000
set echo on
set termout on
set trimspool on

spool vpdtypes.log

select * from v$version;

-- create two test users

drop user test cascade;
drop user test2 cascade;
create user test identified by test;
create user test2 identified by test2;
grant dba to test,test2;

connect test/test

-- this script is to demonstrate the differences in the different
-- policy types
-- STATIC, SHARED_STATIC, CONTEXT_SENSITIVE, SHARED_CONTEXT_SENSITIVE,
-- DYNAMIC

-- first create two test tables

create table test as select * from dba_objects;
create table test2 as select * from test;

execute dbms_stats.gather_table_stats('TEST','TEST');
execute dbms_stats.gather_table_stats('TEST','TEST2');

-- create policy function - potentially returns a different predicate
-- with each execution

CREATE OR REPLACE FUNCTION fn_test_vpd(p_schema       IN VARCHAR2,
                                       p_objname      IN VARCHAR2)
   RETURN VARCHAR2 AS
  
BEGIN
      RETURN 'to_char(CREATED,''SS'')='''||to_char(sysdate,'SS')||'''';
END;
/

-- if function has a second for a gap then predicate is different

select fn_test_vpd('TEST','TEST') from dual;
host sleep 1
select fn_test_vpd('TEST','TEST') from dual;
host sleep 1
select fn_test_vpd('TEST','TEST') from dual;
host sleep 1
select fn_test_vpd('TEST','TEST') from dual;

-- create policy with devault DYNAMIC policy type
-- on both tables

BEGIN
dbms_rls.add_policy(object_schema => 'TEST',
object_name => 'TEST',
policy_name => 'TEST_POLICY',
function_schema =>'TEST',
policy_function => 'FN_TEST_VPD',
statement_types =>'SELECT',
policy_type => dbms_rls.DYNAMIC);

dbms_rls.add_policy(object_schema => 'TEST',
object_name => 'TEST2',
policy_name => 'TEST_POLICY',
function_schema =>'TEST',
policy_function => 'FN_TEST_VPD',
statement_types =>'SELECT',
policy_type => dbms_rls.DYNAMIC);

END;
/

-- try query twice on both tables, both users
-- predicate is different each time so rowcount
-- is different

connect test/test

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

select count(*) from test;
host sleep 1
select count(*) from test;
host sleep 1
select count(*) from test2;
host sleep 1
select count(*) from test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

-- check hard parse count
-- not sure why this is 5 parses

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';

connect test2/test2

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

select count(*) from test.test;
host sleep 1
select count(*) from test.test;
host sleep 1
select count(*) from test.test2;
host sleep 1
select count(*) from test.test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

-- check hard parse count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';

-- change policy to STATIC policy type
-- on both tables

BEGIN

DBMS_RLS.DROP_POLICY(object_schema => 'TEST',
object_name => 'TEST',
policy_name => 'TEST_POLICY'); 

DBMS_RLS.DROP_POLICY(object_schema => 'TEST',
object_name => 'TEST2',
policy_name => 'TEST_POLICY'); 

dbms_rls.add_policy(object_schema => 'TEST',
object_name => 'TEST',
policy_name => 'TEST_POLICY',
function_schema =>'TEST',
policy_function => 'FN_TEST_VPD',
statement_types =>'SELECT',
policy_type => dbms_rls.STATIC);

dbms_rls.add_policy(object_schema => 'TEST',
object_name => 'TEST2',
policy_name => 'TEST_POLICY',
function_schema =>'TEST',
policy_function => 'FN_TEST_VPD',
statement_types =>'SELECT',
policy_type => dbms_rls.STATIC);

END;
/

-- try query twice on both tables, both users
-- predicate is same each time for a given table so rowcount
-- is the same each time that table is queried

connect test/test

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';


select count(*) from test;
host sleep 1
select count(*) from test;
host sleep 1
select count(*) from test2;
host sleep 1
select count(*) from test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';


-- check hard parse count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';


connect test2/test2

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

select count(*) from test.test;
host sleep 1
select count(*) from test.test;
host sleep 1
select count(*) from test.test2;
host sleep 1
select count(*) from test.test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

-- check hard parse count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';

-- change policy to SHARED_STATIC policy type
-- on both tables

BEGIN

DBMS_RLS.DROP_POLICY(object_schema => 'TEST',
object_name => 'TEST',
policy_name => 'TEST_POLICY'); 

DBMS_RLS.DROP_POLICY(object_schema => 'TEST',
object_name => 'TEST2',
policy_name => 'TEST_POLICY'); 

dbms_rls.add_policy(object_schema => 'TEST',
object_name => 'TEST',
policy_name => 'TEST_POLICY',
function_schema =>'TEST',
policy_function => 'FN_TEST_VPD',
statement_types =>'SELECT',
policy_type => dbms_rls.SHARED_STATIC);

dbms_rls.add_policy(object_schema => 'TEST',
object_name => 'TEST2',
policy_name => 'TEST_POLICY',
function_schema =>'TEST',
policy_function => 'FN_TEST_VPD',
statement_types =>'SELECT',
policy_type => dbms_rls.SHARED_STATIC);

END;
/

-- try query twice on both tables, both users
-- predicate is same each time so rowcount is same for all
-- queries - shared between the two tables

connect test/test

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

select count(*) from test;
host sleep 1
select count(*) from test;
host sleep 1
select count(*) from test2;
host sleep 1
select count(*) from test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

-- check hard parse count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';

connect test2/test2

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

select count(*) from test.test;
host sleep 1
select count(*) from test.test;
host sleep 1
select count(*) from test.test2;
host sleep 1
select count(*) from test.test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

-- check hard parse count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';

-- change policy to CONTEXT_SENSITIVE policy type
-- on both tables

BEGIN

DBMS_RLS.DROP_POLICY(object_schema => 'TEST',
object_name => 'TEST',
policy_name => 'TEST_POLICY'); 

DBMS_RLS.DROP_POLICY(object_schema => 'TEST',
object_name => 'TEST2',
policy_name => 'TEST_POLICY'); 

dbms_rls.add_policy(object_schema => 'TEST',
object_name => 'TEST',
policy_name => 'TEST_POLICY',
function_schema =>'TEST',
policy_function => 'FN_TEST_VPD',
statement_types =>'SELECT',
policy_type => dbms_rls.CONTEXT_SENSITIVE);

dbms_rls.add_policy(object_schema => 'TEST',
object_name => 'TEST2',
policy_name => 'TEST_POLICY',
function_schema =>'TEST',
policy_function => 'FN_TEST_VPD',
statement_types =>'SELECT',
policy_type => dbms_rls.CONTEXT_SENSITIVE);

END;
/

-- try query twice on both tables, both users
-- predicate seems to change every time so can't tell difference
-- between this and dynamic

connect test/test

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

select count(*) from test;
host sleep 1
select count(*) from test;
host sleep 1
select count(*) from test2;
host sleep 1
select count(*) from test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

-- check hard parse count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';

connect test2/test2

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

select count(*) from test.test;
host sleep 1
select count(*) from test.test;
host sleep 1
select count(*) from test.test2;
host sleep 1
select count(*) from test.test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

-- check hard parse count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';

-- change policy to SHARED_CONTEXT_SENSITIVE policy type
-- on both tables

BEGIN

DBMS_RLS.DROP_POLICY(object_schema => 'TEST',
object_name => 'TEST',
policy_name => 'TEST_POLICY'); 

DBMS_RLS.DROP_POLICY(object_schema => 'TEST',
object_name => 'TEST2',
policy_name => 'TEST_POLICY'); 

dbms_rls.add_policy(object_schema => 'TEST',
object_name => 'TEST',
policy_name => 'TEST_POLICY',
function_schema =>'TEST',
policy_function => 'FN_TEST_VPD',
statement_types =>'SELECT',
policy_type => dbms_rls.SHARED_CONTEXT_SENSITIVE);

dbms_rls.add_policy(object_schema => 'TEST',
object_name => 'TEST2',
policy_name => 'TEST_POLICY',
function_schema =>'TEST',
policy_function => 'FN_TEST_VPD',
statement_types =>'SELECT',
policy_type => dbms_rls.SHARED_CONTEXT_SENSITIVE);

END;
/

-- try query twice on both tables, both users
-- predicate seems to change every time so can't tell difference
-- between this and dynamic

connect test/test

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

select count(*) from test;
host sleep 1
select count(*) from test;
host sleep 1
select count(*) from test2;
host sleep 1
select count(*) from test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

-- check hard parse count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';

connect test2/test2

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';


select count(*) from test.test;
host sleep 1
select count(*) from test.test;
host sleep 1
select count(*) from test.test2;
host sleep 1
select count(*) from test.test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';


-- check hard parse count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';

-- do another series of tests to show that each
-- login causes a new predicate to be generated

connect test/test

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';


select count(*) from test;
host sleep 1
select count(*) from test;
host sleep 1
select count(*) from test2;
host sleep 1
select count(*) from test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

-- check hard parse count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';


connect test2/test2

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

select count(*) from test.test;
host sleep 1
select count(*) from test.test;
host sleep 1
select count(*) from test.test2;
host sleep 1
select count(*) from test.test2;
host sleep 1

-- execute count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name like 'execute count';

-- check hard parse count

select sn.name,ms.value
from V$MYSTAT ms,V$STATNAME sn
where
ms.STATISTIC#=sn.STATISTIC# and
name = 'parse count (hard)';

spool off
